@@ -58,15 +58,15 @@
 
 namespace aria2 {
 
-/* In that URI Selector, select method returns one of the bests 
- * mirrors for first and reserved connections. For supplementary 
- * ones, it returns mirrors which has not been tested yet, and 
- * if each of them already tested, returns mirrors which has to 
+/* In that URI Selector, select method returns one of the bests
+ * mirrors for first and reserved connections. For supplementary
+ * ones, it returns mirrors which has not been tested yet, and
+ * if each of them already tested, returns mirrors which has to
  * be tested again. Otherwise, it doesn't return anymore mirrors.
  */
 
 AdaptiveURISelector::AdaptiveURISelector
-(const SharedHandle<ServerStatMan>& serverStatMan, RequestGroup* requestGroup)
+(const std::shared_ptr<ServerStatMan>& serverStatMan, RequestGroup* requestGroup)
   : serverStatMan_(serverStatMan),
     requestGroup_(requestGroup)
 {
@@ -87,7 +87,7 @@ std::string AdaptiveURISelector::select
     // failed uris that may succeed with more permissive values
     mayRetryWithIncreasedTimeout(fileEntry);
   }
- 
+
   std::string selected = selectOne(uris);
 
   if(selected != A2STR::NIL)
@@ -106,7 +106,7 @@ void AdaptiveURISelector::mayRetryWithIncreasedTimeout(FileEntry* fileEntry)
   std::deque<URIResult> timeouts;
   fileEntry->extractURIResult(timeouts, error_code::TIME_OUT);
   std::transform(timeouts.begin(), timeouts.end(), std::back_inserter(uris),
-                 std::mem_fun_ref(&URIResult::getURI));
+                 std::mem_fn(&URIResult::getURI));
 
   if(A2_LOG_DEBUG_ENABLED) {
     for(std::deque<std::string>::const_iterator i = uris.begin(),
@@ -128,12 +128,12 @@ std::string AdaptiveURISelector::selectOne(const std::deque<std::string>& uris)
     const size_t numPieces =
       requestGroup_->getDownloadContext()->getNumPieces();
 
-    bool reservedContext = numPieces > 0 && 
+    bool reservedContext = numPieces > 0 &&
       static_cast<size_t>(nbConnections_) > std::min
       (numPieces,
        static_cast<size_t>(requestGroup_->getNumConcurrentCommand()));
     bool selectBest = numPieces == 0 || reservedContext;
-    
+
     if(numPieces > 0)
       ++nbConnections_;
 
@@ -148,7 +148,7 @@ std::string AdaptiveURISelector::selectOne(const std::deque<std::string>& uris)
         return notTested;
       }
     }
-    
+
     if(!selectBest && nbConnections_ > 1 && nbServerToEvaluate_ > 0) {
       nbServerToEvaluate_--;
       std::string notTested = getFirstNotTestedUri(uris);
@@ -184,7 +184,7 @@ std::string AdaptiveURISelector::getBestMirror
   int max = getMaxDownloadSpeed(uris);
   int min = max-(int)(max*0.25);
   std::deque<std::string> bests = getUrisBySpeed(uris, min);
-  
+
   if (bests.size() < 2) {
     std::string uri = getMaxDownloadSpeedUri(uris);
     A2_LOG_DEBUG(fmt("AdaptiveURISelector: choosing the best mirror :"
@@ -242,7 +242,7 @@ void AdaptiveURISelector::adjustLowestSpeedLimit
 }
 
 namespace {
-int getUriMaxSpeed(SharedHandle<ServerStat> ss)
+int getUriMaxSpeed(std::shared_ptr<ServerStat> ss)
 {
   return std::max(ss->getSingleConnectionAvgSpeed(),
                   ss->getMultiConnectionAvgSpeed());
@@ -263,9 +263,8 @@ std::string AdaptiveURISelector::getMaxDownloadSpeedUri
 {
   int max = -1;
   std::string uri = A2STR::NIL;
-  for(std::deque<std::string>::const_iterator i = uris.begin(),
-        eoi = uris.end(); i != eoi; ++i) {
-    SharedHandle<ServerStat> ss = getServerStats(*i);
+  for(auto i = uris.begin(), eoi = uris.end(); i != eoi; ++i) {
+    std::shared_ptr<ServerStat> ss = getServerStats(*i);
     if(!ss)
       continue;
 
@@ -285,9 +284,8 @@ std::deque<std::string> AdaptiveURISelector::getUrisBySpeed
 (const std::deque<std::string>& uris, int min) const
 {
   std::deque<std::string> bests;
-  for(std::deque<std::string>::const_iterator i = uris.begin(),
-        eoi = uris.end(); i != eoi; ++i) {
-    SharedHandle<ServerStat> ss = getServerStats(*i);
+  for(auto i = uris.begin(), eoi = uris.end(); i != eoi; ++i) {
+    std::shared_ptr<ServerStat> ss = getServerStats(*i);
     if(!ss)
       continue;
     if(ss->getSingleConnectionAvgSpeed() > min ||
@@ -302,7 +300,7 @@ std::string AdaptiveURISelector::selectRandomUri
 (const std::deque<std::string>& uris) const
 {
   int pos = SimpleRandomizer::getInstance()->getRandomNumber(uris.size());
-  std::deque<std::string>::const_iterator i = uris.begin();
+  auto i = uris.begin();
   i = i+pos;
   return *i;
 }
@@ -310,11 +308,10 @@ std::string AdaptiveURISelector::selectRandomUri
 std::string AdaptiveURISelector::getFirstNotTestedUri
 (const std::deque<std::string>& uris) const
 {
-  for(std::deque<std::string>::const_iterator i = uris.begin(),
-        eoi = uris.end(); i != eoi; ++i) {
-    SharedHandle<ServerStat> ss = getServerStats(*i);
+  for (const auto& i : uris) {
+    std::shared_ptr<ServerStat> ss = getServerStats(i);
     if(!ss)
-      return *i;
+      return i;
   }
   return A2STR::NIL;
 }
@@ -324,9 +321,8 @@ std::string AdaptiveURISelector::getFirstToTestUri
 {
   int counter;
   int power;
-  for(std::deque<std::string>::const_iterator i = uris.begin(),
-        eoi = uris.end(); i != eoi; ++i) {
-    SharedHandle<ServerStat> ss = getServerStats(*i);
+  for (const auto& u : uris) {
+    std::shared_ptr<ServerStat> ss = getServerStats(u);
     if(!ss)
       continue;
     counter = ss->getCounter();
@@ -336,20 +332,22 @@ std::string AdaptiveURISelector::getFirstToTestUri
     /* We test the mirror another time if it has not been
      * tested since 2^counter days */
     if(ss->getLastUpdated().difference() > power*24*60*60) {
-      return *i;
+      return u;
     }
   }
   return A2STR::NIL;
 }
 
-SharedHandle<ServerStat> AdaptiveURISelector::getServerStats
+std::shared_ptr<ServerStat> AdaptiveURISelector::getServerStats
 (const std::string& uri) const
 {
-  uri::UriStruct us;
-  if(uri::parse(us, uri)) {
-    return serverStatMan_->find(us.host, us.protocol);
+  uri_split_result us;
+  if(uri_split(&us, uri.c_str()) == 0) {
+    std::string host = uri::getFieldString(us, USR_HOST, uri.c_str());
+    std::string protocol = uri::getFieldString(us, USR_SCHEME, uri.c_str());
+    return serverStatMan_->find(host, protocol);
   } else {
-    return SharedHandle<ServerStat>();
+    return nullptr;
   }
 }
 
@@ -357,9 +355,8 @@ int AdaptiveURISelector::getNbTestedServers
 (const std::deque<std::string>& uris) const
 {
   int counter = 0;
-  for(std::deque<std::string>::const_iterator i = uris.begin(),
-        eoi = uris.end(); i != eoi; ++i) {
-    SharedHandle<ServerStat> ss = getServerStats(*i);
+  for(const auto& u : uris) {
+    std::shared_ptr<ServerStat> ss = getServerStats(u);
     if(!ss)
       ++counter;
   }
