@@ -35,16 +35,20 @@
 #include "BtInterestedMessage.h"
 #include "Peer.h"
 #include "PeerStorage.h"
+#include "SocketBuffer.h"
 
 namespace aria2 {
 
 const char BtInterestedMessage::NAME[] = "interested";
 
-BtInterestedMessage::BtInterestedMessage():ZeroBtMessage(ID, NAME) {}
+BtInterestedMessage::BtInterestedMessage()
+  : ZeroBtMessage(ID, NAME),
+    peerStorage_(nullptr)
+{}
 
 BtInterestedMessage::~BtInterestedMessage() {}
 
-BtInterestedMessage* BtInterestedMessage::create
+std::unique_ptr<BtInterestedMessage> BtInterestedMessage::create
 (const unsigned char* data, size_t dataLength)
 {
   return ZeroBtMessage::create<BtInterestedMessage>(data, dataLength);
@@ -66,12 +70,26 @@ bool BtInterestedMessage::sendPredicate() const
   return !getPeer()->amInterested();
 }
 
-void BtInterestedMessage::onSendComplete() {
-  getPeer()->amInterested(true);
+namespace {
+struct ThisProgressUpdate : public ProgressUpdate {
+  ThisProgressUpdate(const std::shared_ptr<Peer>& peer)
+    : peer(peer) {}
+  virtual void update(size_t length, bool complete) CXX11_OVERRIDE
+  {
+    if(complete) {
+      peer->amInterested(true);
+    }
+  }
+  std::shared_ptr<Peer> peer;
+};
+} // namespace
+
+std::unique_ptr<ProgressUpdate> BtInterestedMessage::getProgressUpdate()
+{
+  return make_unique<ThisProgressUpdate>(getPeer());
 }
 
-void BtInterestedMessage::setPeerStorage
-(const SharedHandle<PeerStorage>& peerStorage)
+void BtInterestedMessage::setPeerStorage(PeerStorage* peerStorage)
 {
   peerStorage_ = peerStorage;
 }
