@@ -35,16 +35,20 @@
 #include "BtNotInterestedMessage.h"
 #include "Peer.h"
 #include "PeerStorage.h"
+#include "SocketBuffer.h"
 
 namespace aria2 {
 
 const char BtNotInterestedMessage::NAME[] = "not interested";
 
-BtNotInterestedMessage::BtNotInterestedMessage():ZeroBtMessage(ID, NAME) {}
+BtNotInterestedMessage::BtNotInterestedMessage()
+  : ZeroBtMessage(ID, NAME),
+    peerStorage_(nullptr)
+{}
 
 BtNotInterestedMessage::~BtNotInterestedMessage() {}
 
-BtNotInterestedMessage* BtNotInterestedMessage::create
+std::unique_ptr<BtNotInterestedMessage> BtNotInterestedMessage::create
 (const unsigned char* data, size_t dataLength)
 {
   return ZeroBtMessage::create<BtNotInterestedMessage>(data, dataLength);
@@ -66,12 +70,26 @@ bool BtNotInterestedMessage::sendPredicate() const
   return getPeer()->amInterested();
 }
 
-void BtNotInterestedMessage::onSendComplete() {
-  getPeer()->amInterested(false);
+namespace {
+struct ThisProgressUpdate : public ProgressUpdate {
+  ThisProgressUpdate(const std::shared_ptr<Peer>& peer)
+    : peer(peer) {}
+  virtual void update(size_t length, bool complete) CXX11_OVERRIDE
+  {
+    if(complete) {
+      peer->amInterested(false);
+    }
+  }
+  std::shared_ptr<Peer> peer;
+};
+} // namespace
+
+std::unique_ptr<ProgressUpdate> BtNotInterestedMessage::getProgressUpdate()
+{
+  return make_unique<ThisProgressUpdate>(getPeer());
 }
 
-void BtNotInterestedMessage::setPeerStorage
-(const SharedHandle<PeerStorage>& peerStorage)
+void BtNotInterestedMessage::setPeerStorage(PeerStorage* peerStorage)
 {
   peerStorage_ = peerStorage;
 }
