@@ -34,14 +34,15 @@
 /* copyright --> */
 #include "BtUnchokeMessage.h"
 #include "Peer.h"
+#include "SocketBuffer.h"
 
 namespace aria2 {
 
-const std::string BtUnchokeMessage::NAME("unchoke");
+const char BtUnchokeMessage::NAME[] = "unchoke";
 
 BtUnchokeMessage::BtUnchokeMessage():ZeroBtMessage(ID, NAME) {}
 
-SharedHandle<BtUnchokeMessage> BtUnchokeMessage::create
+std::unique_ptr<BtUnchokeMessage> BtUnchokeMessage::create
 (const unsigned char* data, size_t dataLength)
 {
   return ZeroBtMessage::create<BtUnchokeMessage>(data, dataLength);
@@ -60,8 +61,23 @@ bool BtUnchokeMessage::sendPredicate() const
   return getPeer()->amChoking();
 }
 
-void BtUnchokeMessage::onSendComplete() {
-  getPeer()->amChoking(false);
+namespace {
+struct ThisProgressUpdate : public ProgressUpdate {
+  ThisProgressUpdate(const std::shared_ptr<Peer>& peer)
+    : peer(peer) {}
+  virtual void update(size_t length, bool complete) CXX11_OVERRIDE
+  {
+    if(complete) {
+      peer->amChoking(false);
+    }
+  }
+  std::shared_ptr<Peer> peer;
+};
+} // namespace
+
+std::unique_ptr<ProgressUpdate> BtUnchokeMessage::getProgressUpdate()
+{
+  return make_unique<ThisProgressUpdate>(getPeer());
 }
 
 } // namespace aria2
