@@ -48,18 +48,20 @@
 #include "SimpleRandomizer.h"
 #include "array_fun.h"
 #include "fmt.h"
+#include "BtRequestMessage.h"
 
 namespace aria2 {
 
 DefaultBtRequestFactory::DefaultBtRequestFactory()
-  : dispatcher_(0),
-    messageFactory_(0),
+  : pieceStorage_(nullptr),
+    dispatcher_(nullptr),
+    messageFactory_(nullptr),
     cuid_(0)
 {}
 
 DefaultBtRequestFactory::~DefaultBtRequestFactory() {}
 
-void DefaultBtRequestFactory::addTargetPiece(const SharedHandle<Piece>& piece)
+void DefaultBtRequestFactory::addTargetPiece(const std::shared_ptr<Piece>& piece)
 {
   pieces_.push_back(piece);
 }
@@ -73,7 +75,7 @@ public:
   AbortCompletedPieceRequest(BtMessageDispatcher* dispatcher):
     dispatcher_(dispatcher) {}
 
-  void operator()(const SharedHandle<Piece>& piece)
+  void operator()(const std::shared_ptr<Piece>& piece)
   {
     if(piece->pieceComplete()) {
       dispatcher_->doAbortOutstandingRequestAction(piece);
@@ -86,12 +88,12 @@ void DefaultBtRequestFactory::removeCompletedPiece() {
   std::for_each(pieces_.begin(), pieces_.end(),
                 AbortCompletedPieceRequest(dispatcher_));
   pieces_.erase(std::remove_if(pieces_.begin(), pieces_.end(),
-                              mem_fun_sh(&Piece::pieceComplete)),
-               pieces_.end());
+                               std::mem_fn(&Piece::pieceComplete)),
+                pieces_.end());
 }
 
 void DefaultBtRequestFactory::removeTargetPiece
-(const SharedHandle<Piece>& piece)
+(const std::shared_ptr<Piece>& piece)
 {
   pieces_.erase(std::remove_if(pieces_.begin(), pieces_.end(),
                                derefEqual(piece)),
@@ -103,19 +105,19 @@ void DefaultBtRequestFactory::removeTargetPiece
 namespace {
 class ProcessChokedPiece {
 private:
-  SharedHandle<Peer> peer_;
-  SharedHandle<PieceStorage> pieceStorage_;
+  std::shared_ptr<Peer> peer_;
+  PieceStorage* pieceStorage_;
   cuid_t cuid_;
 public:
-  ProcessChokedPiece(const SharedHandle<Peer>& peer,
-                     const SharedHandle<PieceStorage>& pieceStorage,
+  ProcessChokedPiece(const std::shared_ptr<Peer>& peer,
+                     PieceStorage* pieceStorage,
                      cuid_t cuid):
     peer_(peer),
     pieceStorage_(pieceStorage),
     cuid_(cuid)
   {}
 
-  void operator()(const SharedHandle<Piece>& piece)
+  void operator()(const std::shared_ptr<Piece>& piece)
   {
     if(!peer_->isInPeerAllowedIndexSet(piece->getIndex())) {
       pieceStorage_->cancelPiece(piece, cuid_);
@@ -127,11 +129,11 @@ public:
 namespace {
 class FindChokedPiece {
 private:
-  SharedHandle<Peer> peer_;
+  std::shared_ptr<Peer> peer_;
 public:
-  FindChokedPiece(const SharedHandle<Peer>& peer):peer_(peer) {}
+  FindChokedPiece(const std::shared_ptr<Peer>& peer):peer_(peer) {}
 
-  bool operator()(const SharedHandle<Piece>& piece)
+  bool operator()(const std::shared_ptr<Piece>& piece)
   {
     return !peer_->isInPeerAllowedIndexSet(piece->getIndex());
   }
@@ -148,56 +150,57 @@ void DefaultBtRequestFactory::doChokedAction()
 }
 
 void DefaultBtRequestFactory::removeAllTargetPiece() {
-  for(std::deque<SharedHandle<Piece> >::iterator itr = pieces_.begin(),
-        eoi = pieces_.end(); itr != eoi; ++itr) {
-    dispatcher_->doAbortOutstandingRequestAction(*itr);
-    pieceStorage_->cancelPiece(*itr, cuid_);
+  for(auto & elem : pieces_) {
+    dispatcher_->doAbortOutstandingRequestAction(elem);
+    pieceStorage_->cancelPiece(elem, cuid_);
   }
   pieces_.clear();
 }
 
-void DefaultBtRequestFactory::createRequestMessages
-(std::vector<SharedHandle<BtMessage> >& requests, size_t max)
+std::vector<std::unique_ptr<BtRequestMessage>>
+DefaultBtRequestFactory::createRequestMessages(size_t max, bool endGame)
 {
-  if(requests.size() >= max) {
-    return;
+  if(endGame) {
+    return createRequestMessagesOnEndGame(max);
   }
+  auto requests = std::vector<std::unique_ptr<BtRequestMessage>>{};
   size_t getnum = max-requests.size();
-  std::vector<size_t> blockIndexes;
+  auto blockIndexes = std::vector<size_t>{};
   blockIndexes.reserve(getnum);
-  for(std::deque<SharedHandle<Piece> >::iterator itr = pieces_.begin(),
-        eoi = pieces_.end(); itr != eoi && getnum; ++itr) {
-    SharedHandle<Piece>& piece = *itr;
+  for(auto itr = std::begin(pieces_), eoi = std::end(pieces_);
+      itr != eoi && getnum; ++itr) {
+    auto& piece = *itr;
     if(piece->getMissingUnusedBlockIndex(blockIndexes, getnum)) {
       getnum -= blockIndexes.size();
-      for(std::vector<size_t>::const_iterator i = blockIndexes.begin(),
-            eoi2 = blockIndexes.end(); i != eoi2; ++i) {
+      for(auto i = std::begin(blockIndexes), eoi2 = std::end(blockIndexes);
+          i != eoi2; ++i) {
         A2_LOG_DEBUG
           (fmt("Creating RequestMessage index=%lu, begin=%u,"
                " blockIndex=%lu",
                static_cast<unsigned long>(piece->getIndex()),
                static_cast<unsigned int>((*i)*piece->getBlockLength()),
                static_cast<unsigned long>(*i)));
-        requests.push_back
-          (messageFactory_->createRequestMessage(piece, *i));
+        requests.push_back(messageFactory_->createRequestMessage(piece, *i));
       }
       blockIndexes.clear();
     }
   }
+  return requests;
 }
 
-void DefaultBtRequestFactory::createRequestMessagesOnEndGame
-(std::vector<SharedHandle<BtMessage> >& requests, size_t max)
+std::vector<std::unique_ptr<BtRequestMessage>>
+DefaultBtRequestFactory::createRequestMessagesOnEndGame(size_t max)
 {
-  for(std::deque<SharedHandle<Piece> >::iterator itr = pieces_.begin(),
-        eoi = pieces_.end(); itr != eoi && requests.size() < max; ++itr) {
-    SharedHandle<Piece>& piece = *itr;
+  auto requests = std::vector<std::unique_ptr<BtRequestMessage>>{};
+  for(auto itr = std::begin(pieces_), eoi = std::end(pieces_);
+      itr != eoi && requests.size() < max; ++itr) {
+    auto& piece = *itr;
     const size_t mislen = piece->getBitfieldLength();
-    array_ptr<unsigned char> misbitfield(new unsigned char[mislen]);
+    auto misbitfield = make_unique<unsigned char[]>(mislen);
 
-    piece->getAllMissingBlockIndexes(misbitfield, mislen);
+    piece->getAllMissingBlockIndexes(misbitfield.get(), mislen);
 
-    std::vector<size_t> missingBlockIndexes;
+    auto missingBlockIndexes = std::vector<size_t>{};
     size_t blockIndex = 0;
     for(size_t i = 0; i < mislen; ++i) {
       unsigned char bits = misbitfield[i];
@@ -208,12 +211,13 @@ void DefaultBtRequestFactory::createRequestMessagesOnEndGame
         }
       }
     }
-    std::random_shuffle(missingBlockIndexes.begin(), missingBlockIndexes.end(),
-                        *(SimpleRandomizer::getInstance().get()));
-    for(std::vector<size_t>::const_iterator bitr = missingBlockIndexes.begin(),
-          eoi2 = missingBlockIndexes.end();
+    std::random_shuffle(std::begin(missingBlockIndexes),
+                        std::end(missingBlockIndexes),
+                        *SimpleRandomizer::getInstance());
+    for(auto bitr = std::begin(missingBlockIndexes),
+          eoi2 = std::end(missingBlockIndexes);
         bitr != eoi2 && requests.size() < max; ++bitr) {
-      const size_t& blockIndex = *bitr;
+      size_t blockIndex = *bitr;
       if(!dispatcher_->isOutstandingRequest(piece->getIndex(),
                                            blockIndex)) {
         A2_LOG_DEBUG
@@ -227,6 +231,7 @@ void DefaultBtRequestFactory::createRequestMessagesOnEndGame
       }
     }
   }
+  return requests;
 }
 
 namespace {
@@ -242,7 +247,7 @@ public:
     return numMissingBlock_;
   }
 
-  void operator()(const SharedHandle<Piece>& piece)
+  void operator()(const std::shared_ptr<Piece>& piece)
   {
     numMissingBlock_ += piece->countMissingBlock();
   }
@@ -255,20 +260,22 @@ size_t DefaultBtRequestFactory::countMissingBlock()
                        CountMissingBlock()).getNumMissingBlock();
 }
 
-void DefaultBtRequestFactory::getTargetPieceIndexes
-(std::vector<size_t>& indexes) const
+std::vector<size_t>
+DefaultBtRequestFactory::getTargetPieceIndexes() const
 {
-  std::transform(pieces_.begin(), pieces_.end(), std::back_inserter(indexes),
-                 mem_fun_sh(&Piece::getIndex));
+  auto res = std::vector<size_t>{};
+  res.reserve(pieces_.size());
+  std::transform(std::begin(pieces_), std::end(pieces_),
+                 std::back_inserter(res), std::mem_fn(&Piece::getIndex));
+  return res;
 }
 
-void DefaultBtRequestFactory::setPieceStorage
-(const SharedHandle<PieceStorage>& pieceStorage)
+void DefaultBtRequestFactory::setPieceStorage(PieceStorage* pieceStorage)
 {
   pieceStorage_ = pieceStorage;
 }
 
-void DefaultBtRequestFactory::setPeer(const SharedHandle<Peer>& peer)
+void DefaultBtRequestFactory::setPeer(const std::shared_ptr<Peer>& peer)
 {
   peer_ = peer;
 }
