@@ -74,7 +74,7 @@
 namespace aria2 {
 
 DefaultPieceStorage::DefaultPieceStorage
-(const SharedHandle<DownloadContext>& downloadContext, const Option* option)
+(const std::shared_ptr<DownloadContext>& downloadContext, const Option* option)
  : downloadContext_(downloadContext),
    bitfieldMan_(new BitfieldMan(downloadContext->getPieceLength(),
                                 downloadContext->getTotalLength())),
@@ -83,17 +83,20 @@ DefaultPieceStorage::DefaultPieceStorage
    endGamePieceNum_(END_GAME_PIECE_NUM),
    option_(option),
    pieceStatMan_(new PieceStatMan(downloadContext->getNumPieces(), true)),
-   pieceSelector_(new RarestPieceSelector(pieceStatMan_)),
-   wrDiskCache_(0)
+   pieceSelector_(make_unique<RarestPieceSelector>(pieceStatMan_)),
+   wrDiskCache_(nullptr)
 {
   const std::string& pieceSelectorOpt =
     option_->get(PREF_STREAM_PIECE_SELECTOR);
   if(pieceSelectorOpt.empty() || pieceSelectorOpt == A2_V_DEFAULT) {
-    streamPieceSelector_.reset(new DefaultStreamPieceSelector(bitfieldMan_));
+    streamPieceSelector_ = make_unique<DefaultStreamPieceSelector>
+      (bitfieldMan_);
   } else if(pieceSelectorOpt == V_INORDER) {
-    streamPieceSelector_.reset(new InorderStreamPieceSelector(bitfieldMan_));
+    streamPieceSelector_ = make_unique<InorderStreamPieceSelector>
+      (bitfieldMan_);
   } else if(pieceSelectorOpt == A2_V_GEOM) {
-    streamPieceSelector_.reset(new GeomStreamPieceSelector(bitfieldMan_, 1.5));
+    streamPieceSelector_ = make_unique<GeomStreamPieceSelector>
+      (bitfieldMan_, 1.5);
   }
 }
 
@@ -102,12 +105,12 @@ DefaultPieceStorage::~DefaultPieceStorage()
   delete bitfieldMan_;
 }
 
-SharedHandle<Piece> DefaultPieceStorage::checkOutPiece
+std::shared_ptr<Piece> DefaultPieceStorage::checkOutPiece
 (size_t index, cuid_t cuid)
 {
   bitfieldMan_->setUseBit(index);
 
-  SharedHandle<Piece> piece = findUsedPiece(index);
+  std::shared_ptr<Piece> piece = findUsedPiece(index);
   if(!piece) {
     piece.reset(new Piece(index, bitfieldMan_->getBlockLength(index)));
 #ifdef ENABLE_MESSAGE_DIGEST
@@ -133,9 +136,9 @@ SharedHandle<Piece> DefaultPieceStorage::checkOutPiece
  * Newly instantiated piece is not added to usedPieces.
  * Because it is waste of memory and there is no chance to use them later.
  */
-SharedHandle<Piece> DefaultPieceStorage::getPiece(size_t index)
+std::shared_ptr<Piece> DefaultPieceStorage::getPiece(size_t index)
 {
-  SharedHandle<Piece> piece;
+  std::shared_ptr<Piece> piece;
   if(index <= bitfieldMan_->getMaxIndex()) {
     piece = findUsedPiece(index);
     if(!piece) {
@@ -148,16 +151,16 @@ SharedHandle<Piece> DefaultPieceStorage::getPiece(size_t index)
   return piece;
 }
 
-void DefaultPieceStorage::addUsedPiece(const SharedHandle<Piece>& piece)
+void DefaultPieceStorage::addUsedPiece(const std::shared_ptr<Piece>& piece)
 {
   usedPieces_.insert(piece);
   A2_LOG_DEBUG(fmt("usedPieces_.size()=%lu",
                    static_cast<unsigned long>(usedPieces_.size())));
 }
 
-SharedHandle<Piece> DefaultPieceStorage::findUsedPiece(size_t index) const
+std::shared_ptr<Piece> DefaultPieceStorage::findUsedPiece(size_t index) const
 {
-  SharedHandle<Piece> p(new Piece());
+  std::shared_ptr<Piece> p(new Piece());
   p->setIndex(index);
 
   UsedPieceSet::iterator i = usedPieces_.find(p);
@@ -171,26 +174,26 @@ SharedHandle<Piece> DefaultPieceStorage::findUsedPiece(size_t index) const
 
 #ifdef ENABLE_BITTORRENT
 
-bool DefaultPieceStorage::hasMissingPiece(const SharedHandle<Peer>& peer)
+bool DefaultPieceStorage::hasMissingPiece(const std::shared_ptr<Peer>& peer)
 {
   return bitfieldMan_->hasMissingPiece(peer->getBitfield(),
                                        peer->getBitfieldLength());
 }
 
 void DefaultPieceStorage::getMissingPiece
-(std::vector<SharedHandle<Piece> >& pieces,
+(std::vector<std::shared_ptr<Piece> >& pieces,
  size_t minMissingBlocks,
  const unsigned char* bitfield,
  size_t length,
  cuid_t cuid)
 {
   const size_t mislen = bitfieldMan_->getBitfieldLength();
-  array_ptr<unsigned char> misbitfield(new unsigned char[mislen]);
+  auto misbitfield = make_unique<unsigned char[]>(mislen);
   size_t blocks = bitfieldMan_->countBlock();
   size_t misBlock = 0;
   if(isEndGame()) {
     bool r = bitfieldMan_->getAllMissingIndexes
-      (misbitfield, mislen, bitfield, length);
+      (misbitfield.get(), mislen, bitfield, length);
     if(!r) {
       return;
     }
@@ -203,7 +206,7 @@ void DefaultPieceStorage::getMissingPiece
     std::random_shuffle(indexes.begin(), indexes.end());
     for(std::vector<size_t>::const_iterator i = indexes.begin(),
           eoi = indexes.end(); i != eoi && misBlock < minMissingBlocks; ++i) {
-      SharedHandle<Piece> piece = checkOutPiece(*i, cuid);
+      std::shared_ptr<Piece> piece = checkOutPiece(*i, cuid);
       if(piece->getUsedBySegment()) {
         // We don't share piece downloaded via HTTP/FTP
         piece->removeUser(cuid);
@@ -214,15 +217,15 @@ void DefaultPieceStorage::getMissingPiece
     }
   } else {
     bool r = bitfieldMan_->getAllMissingUnusedIndexes
-      (misbitfield, mislen, bitfield, length);
+      (misbitfield.get(), mislen, bitfield, length);
     if(!r) {
       return;
     }
     while(misBlock < minMissingBlocks) {
       size_t index;
-      if(pieceSelector_->select(index, misbitfield, blocks)) {
+      if(pieceSelector_->select(index, misbitfield.get(), blocks)) {
         pieces.push_back(checkOutPiece(index, cuid));
-        bitfield::flipBit(misbitfield, blocks, index);
+        bitfield::flipBit(misbitfield.get(), blocks, index);
         misBlock += pieces.back()->countMissingBlock();
       } else {
         break;
@@ -235,27 +238,27 @@ namespace {
 void unsetExcludedIndexes(BitfieldMan& bitfield,
                           const std::vector<size_t>& excludedIndexes)
 {
+  using namespace std::placeholders;
   std::for_each(excludedIndexes.begin(), excludedIndexes.end(),
-                std::bind1st(std::mem_fun(&BitfieldMan::unsetBit), &bitfield));
+                std::bind(&BitfieldMan::unsetBit, &bitfield, _1));
 }
 } // namespace
 
 void DefaultPieceStorage::createFastIndexBitfield
-(BitfieldMan& bitfield, const SharedHandle<Peer>& peer)
+(BitfieldMan& bitfield, const std::shared_ptr<Peer>& peer)
 {
-  for(std::set<size_t>::const_iterator itr =
-        peer->getPeerAllowedIndexSet().begin(),
-        eoi = peer->getPeerAllowedIndexSet().end(); itr != eoi; ++itr) {
-    if(!bitfieldMan_->isBitSet(*itr) && peer->hasPiece(*itr)) {
-      bitfield.setBit(*itr);
+  const auto& is = peer->getPeerAllowedIndexSet();
+  for(const auto& i: is) {
+    if(!bitfieldMan_->isBitSet(i) && peer->hasPiece(i)) {
+      bitfield.setBit(i);
     }
   }
 }
 
 void DefaultPieceStorage::getMissingPiece
-(std::vector<SharedHandle<Piece> >& pieces,
+(std::vector<std::shared_ptr<Piece> >& pieces,
  size_t minMissingBlocks,
- const SharedHandle<Peer>& peer,
+ const std::shared_ptr<Peer>& peer,
  cuid_t cuid)
 {
   getMissingPiece(pieces, minMissingBlocks,
@@ -265,9 +268,9 @@ void DefaultPieceStorage::getMissingPiece
 
 
 void DefaultPieceStorage::getMissingPiece
-(std::vector<SharedHandle<Piece> >& pieces,
+(std::vector<std::shared_ptr<Piece> >& pieces,
  size_t minMissingBlocks,
- const SharedHandle<Peer>& peer,
+ const std::shared_ptr<Peer>& peer,
  const std::vector<size_t>& excludedIndexes,
  cuid_t cuid)
 {
@@ -281,9 +284,9 @@ void DefaultPieceStorage::getMissingPiece
 }
 
 void DefaultPieceStorage::getMissingFastPiece
-(std::vector<SharedHandle<Piece> >& pieces,
+(std::vector<std::shared_ptr<Piece> >& pieces,
  size_t minMissingBlocks,
- const SharedHandle<Peer>& peer,
+ const std::shared_ptr<Peer>& peer,
  cuid_t cuid)
 {
   if(peer->isFastExtensionEnabled() && peer->countPeerAllowedIndexSet() > 0) {
@@ -298,9 +301,9 @@ void DefaultPieceStorage::getMissingFastPiece
 }
 
 void DefaultPieceStorage::getMissingFastPiece
-(std::vector<SharedHandle<Piece> >& pieces,
+(std::vector<std::shared_ptr<Piece> >& pieces,
  size_t minMissingBlocks,
- const SharedHandle<Peer>& peer,
+ const std::shared_ptr<Peer>& peer,
  const std::vector<size_t>& excludedIndexes,
  cuid_t cuid)
 {
@@ -316,56 +319,56 @@ void DefaultPieceStorage::getMissingFastPiece
   }
 }
 
-SharedHandle<Piece>
+std::shared_ptr<Piece>
 DefaultPieceStorage::getMissingPiece
-(const SharedHandle<Peer>& peer,
+(const std::shared_ptr<Peer>& peer,
  cuid_t cuid)
 {
-  std::vector<SharedHandle<Piece> > pieces;
+  std::vector<std::shared_ptr<Piece> > pieces;
   getMissingPiece(pieces, 1, peer, cuid);
   if(pieces.empty()) {
-    return SharedHandle<Piece>();
+    return nullptr;
   } else {
     return pieces.front();
   }
 }
 
-SharedHandle<Piece> DefaultPieceStorage::getMissingPiece
-(const SharedHandle<Peer>& peer,
+std::shared_ptr<Piece> DefaultPieceStorage::getMissingPiece
+(const std::shared_ptr<Peer>& peer,
  const std::vector<size_t>& excludedIndexes,
  cuid_t cuid)
 {
-  std::vector<SharedHandle<Piece> > pieces;
+  std::vector<std::shared_ptr<Piece> > pieces;
   getMissingPiece(pieces, 1, peer, excludedIndexes, cuid);
   if(pieces.empty()) {
-    return SharedHandle<Piece>();
+    return nullptr;
   } else {
     return pieces.front();
   }
 }
 
-SharedHandle<Piece> DefaultPieceStorage::getMissingFastPiece
-(const SharedHandle<Peer>& peer,
+std::shared_ptr<Piece> DefaultPieceStorage::getMissingFastPiece
+(const std::shared_ptr<Peer>& peer,
  cuid_t cuid)
 {
-  std::vector<SharedHandle<Piece> > pieces;
+  std::vector<std::shared_ptr<Piece> > pieces;
   getMissingFastPiece(pieces, 1, peer, cuid);
   if(pieces.empty()) {
-    return SharedHandle<Piece>();
+    return nullptr;
   } else {
     return pieces.front();
   }
 }
 
-SharedHandle<Piece> DefaultPieceStorage::getMissingFastPiece
-(const SharedHandle<Peer>& peer,
+std::shared_ptr<Piece> DefaultPieceStorage::getMissingFastPiece
+(const std::shared_ptr<Peer>& peer,
  const std::vector<size_t>& excludedIndexes,
  cuid_t cuid)
 {
-  std::vector<SharedHandle<Piece> > pieces;
+  std::vector<std::shared_ptr<Piece> > pieces;
   getMissingFastPiece(pieces, 1, peer, excludedIndexes, cuid);
   if(pieces.empty()) {
-    return SharedHandle<Piece>();
+    return nullptr;
   } else {
     return pieces.front();
   }
@@ -379,7 +382,7 @@ bool DefaultPieceStorage::hasMissingUnusedPiece()
   return bitfieldMan_->getFirstMissingUnusedIndex(index);
 }
 
-SharedHandle<Piece> DefaultPieceStorage::getMissingPiece
+std::shared_ptr<Piece> DefaultPieceStorage::getMissingPiece
 (size_t minSplitSize,
  const unsigned char* ignoreBitfield,
  size_t length,
@@ -390,22 +393,22 @@ SharedHandle<Piece> DefaultPieceStorage::getMissingPiece
      (index, minSplitSize, ignoreBitfield, length)) {
     return checkOutPiece(index, cuid);
   } else {
-    return SharedHandle<Piece>();
+    return nullptr;
   }
 }
 
-SharedHandle<Piece> DefaultPieceStorage::getMissingPiece
+std::shared_ptr<Piece> DefaultPieceStorage::getMissingPiece
 (size_t index,
  cuid_t cuid)
 {
   if(hasPiece(index) || isPieceUsed(index)) {
-    return SharedHandle<Piece>();
+    return nullptr;
   } else {
     return checkOutPiece(index, cuid);
   }
 }
 
-void DefaultPieceStorage::deleteUsedPiece(const SharedHandle<Piece>& piece)
+void DefaultPieceStorage::deleteUsedPiece(const std::shared_ptr<Piece>& piece)
 {
   if(!piece) {
     return;
@@ -434,7 +437,7 @@ void DefaultPieceStorage::deleteUsedPiece(const SharedHandle<Piece>& piece)
 //   size_t deleted = 0;
 //   for(Pieces::iterator itr = usedPieces.begin();
 //       itr != usedPieces.end() && deleted < delNum;) {
-//     SharedHandle<Piece>& piece = *itr;
+//     std::shared_ptr<Piece>& piece = *itr;
 //     if(!bitfieldMan->isUseBitSet(piece->getIndex()) &&
 //        piece->countCompleteBlock() <= piece->countBlock()*(fillRate/100.0)) {
 //       logger->info(MSG_DELETING_USED_PIECE,
@@ -450,7 +453,7 @@ void DefaultPieceStorage::deleteUsedPiece(const SharedHandle<Piece>& piece)
 //   return deleted;
 // }
 
-void DefaultPieceStorage::completePiece(const SharedHandle<Piece>& piece)
+void DefaultPieceStorage::completePiece(const std::shared_ptr<Piece>& piece)
 {
   if(!piece) {
     return;
@@ -477,9 +480,7 @@ void DefaultPieceStorage::completePiece(const SharedHandle<Piece>& piece)
     }
 #ifdef ENABLE_BITTORRENT
     if(downloadContext_->hasAttribute(CTX_ATTR_BT)) {
-      SharedHandle<TorrentAttribute> torrentAttrs =
-        bittorrent::getTorrentAttrs(downloadContext_);
-      if(!torrentAttrs->metadata.empty()) {
+      if(!bittorrent::getTorrentAttrs(downloadContext_)->metadata.empty()) {
 #ifdef __MINGW32__
         // On Windows, if aria2 opens files with GENERIC_WRITE access
         // right, some programs cannot open them aria2 is seeding. To
@@ -494,7 +495,7 @@ void DefaultPieceStorage::completePiece(const SharedHandle<Piece>& piece)
         util::executeHookByOptName(downloadContext_->getOwnerRequestGroup(),
                                    option_, PREF_ON_BT_DOWNLOAD_COMPLETE);
         SingletonHolder<Notifier>::instance()->
-          notifyDownloadEvent(Notifier::ON_BT_DOWNLOAD_COMPLETE,
+          notifyDownloadEvent(EVENT_ON_BT_DOWNLOAD_COMPLETE,
                               downloadContext_->getOwnerRequestGroup());
       }
     }
@@ -509,7 +510,7 @@ bool DefaultPieceStorage::isSelectiveDownloadingMode()
 
 // not unittested
 void DefaultPieceStorage::cancelPiece
-(const SharedHandle<Piece>& piece, cuid_t cuid)
+(const std::shared_ptr<Piece>& piece, cuid_t cuid)
 {
   if(!piece) {
     return;
@@ -565,9 +566,8 @@ int64_t DefaultPieceStorage::getFilteredCompletedLength()
 int64_t DefaultPieceStorage::getInFlightPieceCompletedLength() const
 {
   int64_t len = 0;
-  for(UsedPieceSet::const_iterator i = usedPieces_.begin(),
-        eoi = usedPieces_.end(); i != eoi; ++i) {
-    len += (*i)->getCompletedLength();
+  for(auto & elem : usedPieces_) {
+    len += elem->getCompletedLength();
   }
   return len;
 }
@@ -575,10 +575,9 @@ int64_t DefaultPieceStorage::getInFlightPieceCompletedLength() const
 int64_t DefaultPieceStorage::getInFlightPieceFilteredCompletedLength() const
 {
   int64_t len = 0;
-  for(UsedPieceSet::const_iterator i = usedPieces_.begin(),
-        eoi = usedPieces_.end(); i != eoi; ++i) {
-    if(bitfieldMan_->isFilterBitSet((*i)->getIndex())) {
-      len += (*i)->getCompletedLength();
+  for(auto & elem : usedPieces_) {
+    if(bitfieldMan_->isFilterBitSet(elem->getIndex())) {
+      len += elem->getCompletedLength();
     }
   }
   return len;
@@ -587,13 +586,11 @@ int64_t DefaultPieceStorage::getInFlightPieceFilteredCompletedLength() const
 // not unittested
 void DefaultPieceStorage::setupFileFilter()
 {
-  const std::vector<SharedHandle<FileEntry> >& fileEntries =
+  const std::vector<std::shared_ptr<FileEntry> >& fileEntries =
     downloadContext_->getFileEntries();
   bool allSelected = true;
-  for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
-        fileEntries.begin(), eoi = fileEntries.end();
-      i != eoi; ++i) {
-    if(!(*i)->isRequested()) {
+  for(auto & e : fileEntries) {
+    if(!e->isRequested()) {
       allSelected = false;
       break;
     }
@@ -601,10 +598,9 @@ void DefaultPieceStorage::setupFileFilter()
   if(allSelected) {
     return;
   }
-  for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
-        fileEntries.begin(), eoi = fileEntries.end(); i != eoi; ++i) {
-    if((*i)->isRequested()) {
-      bitfieldMan_->addFilter((*i)->getOffset(), (*i)->getLength());
+  for(auto & e: fileEntries) {
+    if(e->isRequested()) {
+      bitfieldMan_->addFilter(e->getOffset(), e->getLength());
     }
   }
   bitfieldMan_->enableFilter();
@@ -635,25 +631,24 @@ void DefaultPieceStorage::initStorage()
 {
   if(downloadContext_->getFileEntries().size() == 1) {
     A2_LOG_DEBUG("Instantiating DirectDiskAdaptor");
-    DirectDiskAdaptor* directDiskAdaptor(new DirectDiskAdaptor());
+    auto directDiskAdaptor = make_unique<DirectDiskAdaptor>();
     directDiskAdaptor->setTotalLength(downloadContext_->getTotalLength());
     directDiskAdaptor->setFileEntries
       (downloadContext_->getFileEntries().begin(),
        downloadContext_->getFileEntries().end());
 
-    SharedHandle<DiskWriter> writer =
-      diskWriterFactory_->newDiskWriter(directDiskAdaptor->getFilePath());
-    directDiskAdaptor->setDiskWriter(writer);
-    diskAdaptor_.reset(directDiskAdaptor);
+    directDiskAdaptor->setDiskWriter
+      (diskWriterFactory_->newDiskWriter(directDiskAdaptor->getFilePath()));
+    diskAdaptor_ = std::move(directDiskAdaptor);
   } else {
     A2_LOG_DEBUG("Instantiating MultiDiskAdaptor");
-    MultiDiskAdaptor* multiDiskAdaptor(new MultiDiskAdaptor());
+    auto multiDiskAdaptor = make_unique<MultiDiskAdaptor>();
     multiDiskAdaptor->setFileEntries(downloadContext_->getFileEntries().begin(),
                                      downloadContext_->getFileEntries().end());
     multiDiskAdaptor->setPieceLength(downloadContext_->getPieceLength());
     multiDiskAdaptor->setMaxOpenFiles
       (option_->getAsInt(PREF_BT_MAX_OPEN_FILES));
-    diskAdaptor_.reset(multiDiskAdaptor);
+    diskAdaptor_ = std::move(multiDiskAdaptor);
   }
   if(option_->get(PREF_FILE_ALLOCATION) == V_FALLOC) {
     diskAdaptor_->setFileAllocationMethod(DiskAdaptor::FILE_ALLOC_FALLOC);
@@ -679,7 +674,7 @@ const unsigned char* DefaultPieceStorage::getBitfield()
   return bitfieldMan_->getBitfield();
 }
 
-SharedHandle<DiskAdaptor> DefaultPieceStorage::getDiskAdaptor() {
+std::shared_ptr<DiskAdaptor> DefaultPieceStorage::getDiskAdaptor() {
   return diskAdaptor_;
 }
 
@@ -696,12 +691,11 @@ void DefaultPieceStorage::flushWrDiskCacheEntry()
   // UsedPieceSet is sorted by piece index. It means we can flush
   // cache by non-decreasing offset, which is good to reduce disk seek
   // unless the file is heavily fragmented.
-  for(UsedPieceSet::const_iterator i = usedPieces_.begin(),
-        eoi = usedPieces_.end(); i != eoi; ++i) {
-    WrDiskCacheEntry* ce = (*i)->getWrDiskCacheEntry();
+  for(auto & piece : usedPieces_) {
+    auto ce = piece->getWrDiskCacheEntry();
     if(ce) {
-      (*i)->flushWrCache(wrDiskCache_);
-      (*i)->releaseWrCache(wrDiskCache_);
+      piece->flushWrCache(wrDiskCache_);
+      piece->releaseWrCache(wrDiskCache_);
     }
   }
 }
@@ -725,9 +719,6 @@ DefaultPieceStorage::getAdvertisedPieceIndexes(std::vector<size_t>& indexes,
   for(std::deque<HaveEntry>::const_iterator itr = haves_.begin(),
         eoi = haves_.end(); itr != eoi; ++itr) {
     const HaveEntry& have = *itr;
-    if(have.getCuid() == myCuid) {
-      continue;
-    }
     if(lastCheckTime > have.getRegisteredTime()) {
       break;
     }
@@ -755,8 +746,8 @@ public:
 
 void DefaultPieceStorage::removeAdvertisedPiece(time_t elapsed)
 {
-  std::deque<HaveEntry>::iterator itr =
-    std::find_if(haves_.begin(), haves_.end(), FindElapsedHave(elapsed));
+  auto itr = std::find_if(haves_.begin(), haves_.end(),
+                          FindElapsedHave(elapsed));
   if(itr != haves_.end()) {
     A2_LOG_DEBUG(fmt(MSG_REMOVED_HAVE_ENTRY,
                      static_cast<unsigned long>(haves_.end()-itr)));
@@ -784,7 +775,7 @@ void DefaultPieceStorage::markPiecesDone(int64_t length)
     }
     size_t r = (length%bitfieldMan_->getBlockLength())/Piece::BLOCK_LENGTH;
     if(r > 0) {
-      SharedHandle<Piece> p
+      std::shared_ptr<Piece> p
         (new Piece(numPiece, bitfieldMan_->getBlockLength(numPiece)));
 
       for(size_t i = 0; i < r; ++i) {
@@ -808,7 +799,7 @@ void DefaultPieceStorage::markPieceMissing(size_t index)
 }
 
 void DefaultPieceStorage::addInFlightPiece
-(const std::vector<SharedHandle<Piece> >& pieces)
+(const std::vector<std::shared_ptr<Piece> >& pieces)
 {
   usedPieces_.insert(pieces.begin(), pieces.end());
 }
@@ -819,13 +810,13 @@ size_t DefaultPieceStorage::countInFlightPiece()
 }
 
 void DefaultPieceStorage::getInFlightPieces
-(std::vector<SharedHandle<Piece> >& pieces)
+(std::vector<std::shared_ptr<Piece> >& pieces)
 {
   pieces.insert(pieces.end(), usedPieces_.begin(), usedPieces_.end());
 }
 
 void DefaultPieceStorage::setDiskWriterFactory
-(const SharedHandle<DiskWriterFactory>& diskWriterFactory)
+(const std::shared_ptr<DiskWriterFactory>& diskWriterFactory)
 {
   diskWriterFactory_ = diskWriterFactory;
 }
@@ -870,4 +861,15 @@ void DefaultPieceStorage::onDownloadIncomplete()
   streamPieceSelector_->onBitfieldInit();
 }
 
+void DefaultPieceStorage::setPieceSelector
+(std::unique_ptr<PieceSelector> pieceSelector)
+{
+  pieceSelector_ = std::move(pieceSelector);
+}
+
+std::unique_ptr<PieceSelector> DefaultPieceStorage::popPieceSelector()
+{
+  return std::move(pieceSelector_);
+}
+
 } // namespace aria2
