@@ -47,7 +47,7 @@ unsigned char* IndexBtMessage::createMessage()
    * piece index --- index, 4bytes
    * total: 9bytes
    */
-  unsigned char* msg = new unsigned char[MESSAGE_LENGTH];
+  auto   msg = new unsigned char[MESSAGE_LENGTH];
   bittorrent::createPeerMessageString(msg, MESSAGE_LENGTH, 5, getId());
   bittorrent::setIntParam(&msg[5], index_);
   return msg;
@@ -60,9 +60,7 @@ size_t IndexBtMessage::getMessageLength()
 
 std::string IndexBtMessage::toString() const
 {
-  return fmt("%s index=%lu",
-             getName().c_str(),
-             static_cast<unsigned long>(index_));
+  return fmt("%s index=%lu", getName(), static_cast<unsigned long>(index_));
 }
 
 } // namespace aria2
