@@ -210,7 +210,11 @@ std::vector<OptionHandler*> OptionHandlerFactory::createOptionHandlers()
     OptionHandler* op(new UnitNumberOptionHandler
                       (PREF_DISK_CACHE,
                        TEXT_DISK_CACHE,
-                       "0",
+#ifdef DEFAULT_DISK_CACHE
+                       DEFAULT_DISK_CACHE,
+#else
+                       "16M",
+#endif
                        0));
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
@@ -220,8 +224,7 @@ std::vector<OptionHandler*> OptionHandlerFactory::createOptionHandlers()
                       (PREF_CONSOLE_LOG_LEVEL,
                        TEXT_CONSOLE_LOG_LEVEL,
                        V_NOTICE,
-                       std::vector<std::string>
-                       (vbegin(logLevels), vend(logLevels))));
+                       {std::begin(logLevels), std::end(logLevels)}));
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
@@ -282,8 +285,7 @@ std::vector<OptionHandler*> OptionHandlerFactory::createOptionHandlers()
                       (PREF_DOWNLOAD_RESULT,
                        TEXT_DOWNLOAD_RESULT,
                        A2_V_DEFAULT,
-                       A2_V_DEFAULT,
-                       A2_V_FULL));
+                       { A2_V_DEFAULT, A2_V_FULL}));
     op->addTag(TAG_ADVANCED);
     op->setChangeGlobalOption(true);
     handlers.push_back(op);
@@ -328,50 +330,53 @@ std::vector<OptionHandler*> OptionHandlerFactory::createOptionHandlers()
     handlers.push_back(op);
   }
   {
-    std::string params[] = {
-#ifdef HAVE_EPOLL
-      V_EPOLL,
-#endif // HAVE_EPOLL
-#ifdef HAVE_KQUEUE
-      V_KQUEUE,
-#endif // HAVE_KQUEUE
-#ifdef HAVE_PORT_ASSOCIATE
-      V_PORT,
-#endif // HAVE_PORT_ASSOCIATE
-#ifdef HAVE_POLL
-      V_POLL,
-#endif // HAVE_POLL
-      V_SELECT
-    };
     OptionHandler* op(new ParameterOptionHandler
                       (PREF_EVENT_POLL,
                        TEXT_EVENT_POLL,
-#ifdef HAVE_EPOLL
+#if defined(HAVE_EPOLL)
                        V_EPOLL,
-#elif HAVE_KQUEUE
+#elif defined(HAVE_KQUEUE)
                        V_KQUEUE,
-#elif HAVE_PORT_ASSOCIATE
+#elif defined(HAVE_PORT_ASSOCIATE)
                        V_PORT,
-#else
+#elif defined(HAVE_LIBUV)
+                       V_LIBUV,
+#elif defined(HAVE_POLL)
+                       V_POLL,
+#else // defined(HAVE_EPOLL)
                        V_SELECT,
-#endif // !HAVE_EPOLL
-                       std::vector<std::string>
-                       (vbegin(params), vend(params))));
+#endif // defined(HAVE_EPOLL)
+                       {
+#ifdef HAVE_EPOLL
+                         V_EPOLL,
+#endif // HAVE_EPOLL
+#ifdef HAVE_KQUEUE
+                           V_KQUEUE,
+#endif // HAVE_KQUEUE
+#ifdef HAVE_PORT_ASSOCIATE
+                           V_PORT,
+#endif // HAVE_PORT_ASSOCIATE
+#ifdef HAVE_LIBUV
+                           V_LIBUV,
+#endif // HAVE_LIBUV
+#ifdef HAVE_POLL
+                           V_POLL,
+#endif // HAVE_POLL
+                           V_SELECT
+                           }));
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
   {
-    const std::string params[] = { V_NONE, V_PREALLOC, V_TRUNC,
-#ifdef HAVE_SOME_FALLOCATE
-                                   V_FALLOC
-#endif // HAVE_SOME_FALLOCATE
-    };
     OptionHandler* op(new ParameterOptionHandler
                       (PREF_FILE_ALLOCATION,
                        TEXT_FILE_ALLOCATION,
                        V_PREALLOC,
-                       std::vector<std::string>
-                       (vbegin(params), vend(params)),
+                       { V_NONE, V_PREALLOC, V_TRUNC,
+#ifdef HAVE_SOME_FALLOCATE
+                           V_FALLOC
+#endif // HAVE_SOME_FALLOCATE
+                           },
                        'a'));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_FILE);
@@ -477,8 +482,7 @@ std::vector<OptionHandler*> OptionHandlerFactory::createOptionHandlers()
                       (PREF_LOG_LEVEL,
                        TEXT_LOG_LEVEL,
                        V_DEBUG,
-                       std::vector<std::string>
-                       (vbegin(logLevels), vend(logLevels))));
+                       {std::begin(logLevels), std::end(logLevels)}));
     op->addTag(TAG_ADVANCED);
     op->setChangeGlobalOption(true);
     handlers.push_back(op);
@@ -1071,7 +1075,6 @@ std::vector<OptionHandler*> OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
-  OptionHandler* splitHandler;
   {
     OptionHandler* op(new NumberOptionHandler
                       (PREF_SPLIT,
@@ -1085,7 +1088,6 @@ std::vector<OptionHandler*> OptionHandlerFactory::createOptionHandlers()
     op->setInitialOption(true);
     op->setChangeGlobalOption(true);
     op->setChangeOptionForReserved(true);
-    splitHandler = op;
     handlers.push_back(op);
   }
   {
@@ -1102,9 +1104,7 @@ std::vector<OptionHandler*> OptionHandlerFactory::createOptionHandlers()
                       (PREF_STREAM_PIECE_SELECTOR,
                        TEXT_STREAM_PIECE_SELECTOR,
                        A2_V_DEFAULT,
-                       A2_V_DEFAULT,
-                       V_INORDER,
-                       A2_V_GEOM));
+                       { A2_V_DEFAULT, V_INORDER, A2_V_GEOM }));
     op->addTag(TAG_FTP);
     op->addTag(TAG_HTTP);
     op->setInitialOption(true);
@@ -1127,13 +1127,11 @@ std::vector<OptionHandler*> OptionHandlerFactory::createOptionHandlers()
     handlers.push_back(op);
   }
   {
-    const std::string params[] = { V_INORDER, V_FEEDBACK, V_ADAPTIVE };
     OptionHandler* op(new ParameterOptionHandler
                       (PREF_URI_SELECTOR,
                        TEXT_URI_SELECTOR,
                        V_FEEDBACK,
-                       std::vector<std::string>
-                       (vbegin(params), vend(params))));
+                       { V_INORDER, V_FEEDBACK, V_ADAPTIVE }));
     op->addTag(TAG_FTP);
     op->addTag(TAG_HTTP);
     op->setInitialOption(true);
@@ -1146,7 +1144,11 @@ std::vector<OptionHandler*> OptionHandlerFactory::createOptionHandlers()
     OptionHandler* op(new DefaultOptionHandler
                       (PREF_CA_CERTIFICATE,
                        TEXT_CA_CERTIFICATE,
+#ifdef CA_BUNDLE
                        CA_BUNDLE,
+#else
+                       "",
+#endif
                        PATH_TO_FILE));
     op->addTag(TAG_HTTP);
     op->addTag(TAG_HTTPS);
@@ -1346,7 +1348,7 @@ std::vector<OptionHandler*> OptionHandlerFactory::createOptionHandlers()
     OptionHandler* op(new DefaultOptionHandler
                       (PREF_USER_AGENT,
                        TEXT_USER_AGENT,
-                       "aria2/"PACKAGE_VERSION,
+                       "aria2/" PACKAGE_VERSION,
                        A2STR::NIL,
                        OptionHandler::REQ_ARG,
                        'U'));
@@ -1399,7 +1401,7 @@ std::vector<OptionHandler*> OptionHandlerFactory::createOptionHandlers()
                       (PREF_FTP_TYPE,
                        TEXT_FTP_TYPE,
                        V_BINARY,
-                       V_BINARY, V_ASCII));
+                       { V_BINARY, V_ASCII }));
     op->addTag(TAG_FTP);
     op->setInitialOption(true);
     op->setChangeGlobalOption(true);
@@ -1596,7 +1598,7 @@ std::vector<OptionHandler*> OptionHandlerFactory::createOptionHandlers()
                       (PREF_PROXY_METHOD,
                        TEXT_PROXY_METHOD,
                        V_GET,
-                       V_GET, V_TUNNEL));
+                       { V_GET, V_TUNNEL }));
     op->addTag(TAG_FTP);
     op->addTag(TAG_HTTP);
     op->setInitialOption(true);
@@ -1745,7 +1747,7 @@ std::vector<OptionHandler*> OptionHandlerFactory::createOptionHandlers()
                       (PREF_BT_MIN_CRYPTO_LEVEL,
                        TEXT_BT_MIN_CRYPTO_LEVEL,
                        V_PLAIN,
-                       V_PLAIN, V_ARC4));
+                       { V_PLAIN, V_ARC4 }));
     op->addTag(TAG_BITTORRENT);
     op->setInitialOption(true);
     op->setChangeGlobalOption(true);
@@ -2015,7 +2017,7 @@ std::vector<OptionHandler*> OptionHandlerFactory::createOptionHandlers()
                       (PREF_FOLLOW_TORRENT,
                        TEXT_FOLLOW_TORRENT,
                        A2_V_TRUE,
-                       A2_V_TRUE, V_MEM, A2_V_FALSE));
+                       { A2_V_TRUE, V_MEM, A2_V_FALSE }));
     op->addTag(TAG_BITTORRENT);
     op->setInitialOption(true);
     op->setChangeGlobalOption(true);
@@ -2143,7 +2145,7 @@ std::vector<OptionHandler*> OptionHandlerFactory::createOptionHandlers()
                       (PREF_FOLLOW_METALINK,
                        TEXT_FOLLOW_METALINK,
                        A2_V_TRUE,
-                       A2_V_TRUE, V_MEM, A2_V_FALSE));
+                       { A2_V_TRUE, V_MEM, A2_V_FALSE }));
     op->addTag(TAG_METALINK);
     op->setInitialOption(true);
     op->setChangeGlobalOption(true);
@@ -2204,13 +2206,11 @@ std::vector<OptionHandler*> OptionHandlerFactory::createOptionHandlers()
     handlers.push_back(op);
   }
   {
-    const std::string params[] = { V_HTTP, V_HTTPS, V_FTP, V_NONE };
     OptionHandler* op(new ParameterOptionHandler
                       (PREF_METALINK_PREFERRED_PROTOCOL,
                        TEXT_METALINK_PREFERRED_PROTOCOL,
                        V_NONE,
-                       std::vector<std::string>
-                       (vbegin(params), vend(params))));
+                       { V_HTTP, V_HTTPS, V_FTP, V_NONE }));
     op->addTag(TAG_METALINK);
     op->setInitialOption(true);
     op->setChangeGlobalOption(true);
