@@ -33,19 +33,27 @@
  */
 /* copyright --> */
 #include "Piece.h"
+
+#include <cassert>
+
 #include "util.h"
 #include "BitfieldMan.h"
 #include "A2STR.h"
 #include "util.h"
 #include "a2functional.h"
+#include "WrDiskCache.h"
+#include "WrDiskCacheEntry.h"
+#include "LogFactory.h"
+#include "fmt.h"
+#include "DiskAdaptor.h"
 #ifdef ENABLE_MESSAGE_DIGEST
 # include "MessageDigest.h"
 #endif // ENABLE_MESSAGE_DIGEST
 
 namespace aria2 {
 
-Piece::Piece():index_(0), length_(0), blockLength_(BLOCK_LENGTH), bitfield_(0),
-               usedBySegment_(false)
+Piece::Piece():index_(0), length_(0), blockLength_(BLOCK_LENGTH), bitfield_(nullptr),
+               usedBySegment_(false), wrCache_(nullptr)
 #ifdef ENABLE_MESSAGE_DIGEST
               , nextBegin_(0)
 #endif // ENABLE_MESSAGE_DIGEST
@@ -56,7 +64,7 @@ Piece::Piece(size_t index, int32_t length, int32_t blockLength)
    length_(length),
    blockLength_(blockLength),
    bitfield_(new BitfieldMan(blockLength_, length)),
-   usedBySegment_(false)
+   usedBySegment_(false), wrCache_(nullptr)
 #ifdef ENABLE_MESSAGE_DIGEST
  ,nextBegin_(0)
 #endif // ENABLE_MESSAGE_DIGEST
@@ -64,6 +72,7 @@ Piece::Piece(size_t index, int32_t length, int32_t blockLength)
 
 Piece::~Piece()
 {
+  delete wrCache_;
   delete bitfield_;
 }
 
@@ -72,9 +81,12 @@ void Piece::completeBlock(size_t blockIndex) {
   bitfield_->unsetUseBit(blockIndex);
 }
 
-void Piece::clearAllBlock() {
+void Piece::clearAllBlock(WrDiskCache* diskCache) {
   bitfield_->clearAllBit();
   bitfield_->clearAllUseBit();
+  if(diskCache && wrCache_) {
+    clearWrCache(diskCache);
+  }
 }
 
 void Piece::setAllBlock() {
@@ -232,6 +244,56 @@ std::string Piece::getDigest()
   }
 }
 
+namespace {
+void updateHashWithRead(MessageDigest* mdctx,
+                        const std::shared_ptr<DiskAdaptor>& adaptor,
+                        int64_t offset, size_t len)
+{
+  const size_t BUFSIZE = 4096;
+  unsigned char buf[BUFSIZE];
+  ldiv_t res = ldiv(len, BUFSIZE);
+  for(int j = 0; j < res.quot; ++j) {
+    ssize_t nread = adaptor->readData(buf, BUFSIZE, offset);
+    if((size_t)nread != BUFSIZE) {
+      throw DL_ABORT_EX(fmt(EX_FILE_READ, "n/a", "data is too short"));
+    }
+    mdctx->update(buf, nread);
+    offset += nread;
+  }
+  if(res.rem) {
+    ssize_t nread = adaptor->readData(buf, res.rem, offset);
+    if(nread != res.rem) {
+      throw DL_ABORT_EX(fmt(EX_FILE_READ, "n/a", "data is too short"));
+    }
+    mdctx->update(buf, nread);
+    offset += nread;
+  }
+}
+} // namespace
+
+std::string Piece::getDigestWithWrCache
+(size_t pieceLength, const std::shared_ptr<DiskAdaptor>& adaptor)
+{
+  auto mdctx = MessageDigest::create(hashType_);
+  int64_t start = static_cast<int64_t>(index_)*pieceLength;
+  int64_t goff = start;
+  if(wrCache_) {
+    const WrDiskCacheEntry::DataCellSet& dataSet = wrCache_->getDataSet();
+    for(WrDiskCacheEntry::DataCellSet::iterator i = dataSet.begin(),
+          eoi = dataSet.end(); i != eoi; ++i) {
+      if(goff < (*i)->goff) {
+        updateHashWithRead(mdctx.get(), adaptor, goff, (*i)->goff - goff);
+      }
+      mdctx->update((*i)->data+(*i)->offset, (*i)->len);
+      goff = (*i)->goff + (*i)->len;
+    }
+    updateHashWithRead(mdctx.get(), adaptor, goff, start+length_-goff);
+  } else {
+    updateHashWithRead(mdctx.get(), adaptor, goff, length_);
+  }
+  return mdctx->digest();
+}
+
 void Piece::destroyHashContext()
 {
   mdctx_.reset();
@@ -257,4 +319,85 @@ void Piece::removeUser(cuid_t cuid)
   users_.erase(std::remove(users_.begin(), users_.end(), cuid), users_.end());
 }
 
+void Piece::initWrCache(WrDiskCache* diskCache,
+                        const std::shared_ptr<DiskAdaptor>& diskAdaptor)
+{
+  if(!diskCache) {
+    return;
+  }
+  assert(wrCache_ == nullptr);
+  wrCache_ = new WrDiskCacheEntry(diskAdaptor);
+  bool rv = diskCache->add(wrCache_);
+  assert(rv);
+}
+
+void Piece::flushWrCache(WrDiskCache* diskCache)
+{
+  if(!diskCache) {
+    return;
+  }
+  assert(wrCache_);
+  ssize_t size = static_cast<ssize_t>(wrCache_->getSize());
+  diskCache->update(wrCache_, -size);
+  wrCache_->writeToDisk();
+}
+
+void Piece::clearWrCache(WrDiskCache* diskCache)
+{
+  if(!diskCache) {
+    return;
+  }
+  assert(wrCache_);
+  ssize_t size = static_cast<ssize_t>(wrCache_->getSize());
+  diskCache->update(wrCache_, -size);
+  wrCache_->clear();
+}
+
+void Piece::updateWrCache(WrDiskCache* diskCache, unsigned char* data,
+                          size_t offset, size_t len, size_t capacity,
+                          int64_t goff)
+{
+  if(!diskCache) {
+    return;
+  }
+  assert(wrCache_);
+  A2_LOG_DEBUG(fmt("updateWrCache entry=%p", wrCache_));
+  auto cell = new WrDiskCacheEntry::DataCell();
+  cell->goff = goff;
+  cell->data = data;
+  cell->offset = offset;
+  cell->len = len;
+  cell->capacity = capacity;
+  bool rv;
+  rv = wrCache_->cacheData(cell);
+  assert(rv);
+  rv = diskCache->update(wrCache_, len);
+  assert(rv);
+}
+
+size_t Piece::appendWrCache(WrDiskCache* diskCache, int64_t goff,
+                            const unsigned char* data, size_t len)
+{
+  if(!diskCache) {
+    return 0;
+  }
+  assert(wrCache_);
+  size_t delta = wrCache_->append(goff, data, len);
+  bool rv;
+  if(delta > 0) {
+    rv = diskCache->update(wrCache_, delta);
+    assert(rv);
+  }
+  return delta;
+}
+
+void Piece::releaseWrCache(WrDiskCache* diskCache)
+{
+  if(diskCache && wrCache_) {
+    diskCache->remove(wrCache_);
+    delete wrCache_;
+    wrCache_ = nullptr;
+  }
+}
+
 } // namespace aria2
