@@ -36,6 +36,7 @@
 
 #include <cstdio>
 #include <iterator>
+#include <set>
 
 #include "RequestGroupMan.h"
 #include "a2functional.h"
@@ -59,26 +60,27 @@
 namespace aria2 {
 
 SessionSerializer::SessionSerializer
-(const SharedHandle<RequestGroupMan>& requestGroupMan):
-  rgman_(requestGroupMan),
-  saveError_(true),
-  saveInProgress_(true),
-  saveWaiting_(true) {}
+(RequestGroupMan* requestGroupMan)
+  : rgman_{requestGroupMan},
+    saveError_{true},
+    saveInProgress_{true},
+    saveWaiting_{true}
+{}
 
 bool SessionSerializer::save(const std::string& filename) const
 {
   std::string tempFilename = filename;
   tempFilename += "__temp";
   {
-    SharedHandle<BufferedFile> fp;
+    std::shared_ptr<IOFile> fp;
 #if HAVE_ZLIB
     if (util::endsWith(filename, ".gz")) {
-      fp.reset(new GZipFile(tempFilename.c_str(), BufferedFile::WRITE));
+      fp.reset(new GZipFile(tempFilename.c_str(), IOFile::WRITE));
     }
     else
 #endif
     {
-     fp.reset(new BufferedFile(tempFilename.c_str(), BufferedFile::WRITE));
+     fp.reset(new BufferedFile(tempFilename.c_str(), IOFile::WRITE));
     }
     if(!*fp) {
       return false;
@@ -93,7 +95,7 @@ bool SessionSerializer::save(const std::string& filename) const
 namespace {
 // Write 1 line of option name/value pair. This function returns true
 // if it succeeds, or false.
-bool writeOptionLine(BufferedFile& fp, const Pref* pref,
+bool writeOptionLine(IOFile& fp, const Pref* pref,
                      const std::string& val)
 {
   size_t prefLen = strlen(pref->k);
@@ -106,13 +108,13 @@ bool writeOptionLine(BufferedFile& fp, const Pref* pref,
 } // namespace
 
 namespace {
-bool writeOption(BufferedFile& fp, const SharedHandle<Option>& op)
+bool writeOption(IOFile& fp, const std::shared_ptr<Option>& op)
 {
-  const SharedHandle<OptionParser>& oparser = OptionParser::getInstance();
+  const std::shared_ptr<OptionParser>& oparser = OptionParser::getInstance();
   for(size_t i = 1, len = option::countOption(); i < len; ++i) {
     const Pref* pref = option::i2p(i);
     const OptionHandler* h = oparser->find(pref);
-    if(h && h->getInitialOption() && op->defined(pref)) {
+    if(h && h->getInitialOption() && op->definedLocal(pref)) {
       if(h->getCumulative()) {
         const std::string& val = op->get(pref);
         std::vector<std::string> v;
@@ -135,6 +137,44 @@ bool writeOption(BufferedFile& fp, const SharedHandle<Option>& op)
 }
 } // namespace
 
+namespace {
+  template<typename T>
+  class Unique {
+    typedef T type;
+    struct PointerCmp {
+      inline bool operator()(const type* x, const type* y) {
+        return *x < *y;
+      }
+    };
+    std::set<const type*, PointerCmp> known;
+  public:
+    inline bool operator()(const type& v) {
+      return known.insert(&v).second;
+    }
+  };
+
+  bool writeUri(IOFile& fp, const std::string& uri)
+  {
+    return fp.write(uri.c_str(), uri.size()) == uri.size() &&
+      fp.write("\t", 1) == 1;
+  }
+
+  template<typename InputIterator, class UnaryPredicate>
+  bool writeUri(IOFile& fp, InputIterator first, InputIterator last,
+                UnaryPredicate& filter)
+  {
+    for(; first != last; ++first) {
+      if (!filter(*first)) {
+        continue;
+      }
+      if(!writeUri(fp, *first)) {
+        return false;
+      }
+    }
+    return true;
+  }
+} // namespace
+
 // The downloads whose followedBy() is empty is persisited with its
 // GID without no problem. For other cases, there are several patterns.
 //
@@ -151,10 +191,10 @@ bool writeOption(BufferedFile& fp, const SharedHandle<Option>& op)
 
 namespace {
 bool writeDownloadResult
-(BufferedFile& fp, std::set<a2_gid_t>& metainfoCache,
- const SharedHandle<DownloadResult>& dr)
+(IOFile& fp, std::set<a2_gid_t>& metainfoCache,
+ const std::shared_ptr<DownloadResult>& dr)
 {
-  const SharedHandle<MetadataInfo>& mi = dr->metadataInfo;
+  const std::shared_ptr<MetadataInfo>& mi = dr->metadataInfo;
   if(dr->belongsTo != 0 || (mi && mi->dataOnly())) {
     return true;
   }
@@ -172,16 +212,26 @@ bool writeDownloadResult
     if(dr->fileEntries.empty()) {
       return true;
     }
-    const SharedHandle<FileEntry>& file = dr->fileEntries[0];
-    std::vector<std::string> uris;
-    file->getUris(uris);
-    if(uris.empty()) {
+    const std::shared_ptr<FileEntry>& file = dr->fileEntries[0];
+    // Don't save download if there are no URIs.
+    const bool hasRemaining = !file->getRemainingUris().empty();
+    const bool hasSpent = !file->getSpentUris().empty();
+    if (!hasRemaining && !hasSpent) {
       return true;
     }
-    for(std::vector<std::string>::const_iterator i = uris.begin(),
-          eoi = uris.end(); i != eoi; ++i) {
-      if (fp.write((*i).c_str(), (*i).size()) != (*i).size() ||
-          fp.write("\t", 1) != 1) {
+
+    // Save spent URIs + remaining URIs. Remove URI in spent URI which
+    // also exists in remaining URIs.
+    {
+      Unique<std::string> unique;
+      if (hasRemaining && !writeUri(fp, file->getRemainingUris().begin(),
+                                    file->getRemainingUris().end(),
+                                    unique)) {
+        return false;
+      }
+      if (hasSpent && !writeUri(fp, file->getSpentUris().begin(),
+                                file->getSpentUris().end(),
+                                unique)) {
         return false;
       }
     }
@@ -212,13 +262,13 @@ bool writeDownloadResult
 }
 } // namespace
 
-bool SessionSerializer::save(BufferedFile& fp) const
+bool SessionSerializer::save(IOFile& fp) const
 {
   std::set<a2_gid_t> metainfoCache;
   const DownloadResultList& results = rgman_->getDownloadResults();
   for(DownloadResultList::const_iterator itr = results.begin(),
         eoi = results.end(); itr != eoi; ++itr) {
-    const SharedHandle<DownloadResult>& dr = *itr;
+    const std::shared_ptr<DownloadResult>& dr = *itr;
     if(dr->result == error_code::FINISHED ||
        dr->result == error_code::REMOVED) {
       if(dr->option->getAsBool(PREF_FORCE_SAVE)) {
@@ -248,8 +298,8 @@ bool SessionSerializer::save(BufferedFile& fp) const
     const RequestGroupList& groups = rgman_->getRequestGroups();
     for(RequestGroupList::const_iterator itr = groups.begin(),
           eoi = groups.end(); itr != eoi; ++itr) {
-      const SharedHandle<RequestGroup>& rg = *itr;
-      SharedHandle<DownloadResult> dr = rg->createDownloadResult();
+      const std::shared_ptr<RequestGroup>& rg = *itr;
+      std::shared_ptr<DownloadResult> dr = rg->createDownloadResult();
       bool stopped = dr->result == error_code::FINISHED ||
         dr->result == error_code::REMOVED;
       if((!stopped && saveInProgress_) ||
@@ -264,8 +314,8 @@ bool SessionSerializer::save(BufferedFile& fp) const
     const RequestGroupList& groups = rgman_->getReservedGroups();
     for(RequestGroupList::const_iterator itr = groups.begin(),
           eoi = groups.end(); itr != eoi; ++itr) {
-      const SharedHandle<RequestGroup>& rg = *itr;
-      SharedHandle<DownloadResult> result = rg->createDownloadResult();
+      const std::shared_ptr<RequestGroup>& rg = *itr;
+      std::shared_ptr<DownloadResult> result = rg->createDownloadResult();
       if(!writeDownloadResult(fp, metainfoCache, result)) {
         return false;
       }
