@@ -41,19 +41,9 @@
 #include <cstdlib>
 #include <utility>
 #include <vector>
+#include <memory>
 
 #include "a2netcompat.h"
-
-#ifdef HAVE_OPENSSL
-// for SSL
-# include <openssl/ssl.h>
-# include <openssl/err.h>
-#endif // HAVE_OPENSSL
-#ifdef HAVE_LIBGNUTLS
-# include <gnutls/gnutls.h>
-#endif // HAVE_LIBGNUTLS
-
-#include "SharedHandle.h"
 #include "a2io.h"
 #include "a2netcompat.h"
 #include "a2time.h"
@@ -62,6 +52,7 @@ namespace aria2 {
 
 #ifdef ENABLE_SSL
 class TLSContext;
+class TLSSession;
 #endif // ENABLE_SSL
 
 class SocketCore {
@@ -86,30 +77,12 @@ private:
 
 #if ENABLE_SSL
   // TLS context for client side
-  static SharedHandle<TLSContext> clTlsContext_;
+  static std::shared_ptr<TLSContext> clTlsContext_;
   // TLS context for server side
-  static SharedHandle<TLSContext> svTlsContext_;
-#endif // ENABLE_SSL
+  static std::shared_ptr<TLSContext> svTlsContext_;
 
-#ifdef HAVE_OPENSSL
-  // for SSL
-  SSL* ssl;
+  std::shared_ptr<TLSSession> tlsSession_;
 
-  int sslHandleEAGAIN(int ret);
-#endif // HAVE_OPENSSL
-#ifdef HAVE_LIBGNUTLS
-  gnutls_session_t sslSession_;
-
-  void gnutlsRecordCheckDirection();
-#endif // HAVE_LIBGNUTLS
-
-  void init();
-
-  void bind(const struct sockaddr* addr, socklen_t addrlen);
-
-  void setSockOpt(int level, int optname, void* optval, socklen_t optlen);
-
-#ifdef ENABLE_SSL
   /**
    * Makes this socket secure. The connection must be established
    * before calling this method.
@@ -119,6 +92,12 @@ private:
   bool tlsHandshake(TLSContext* tlsctx, const std::string& hostname);
 #endif // ENABLE_SSL
 
+  void init();
+
+  void bind(const struct sockaddr* addr, socklen_t addrlen);
+
+  void setSockOpt(int level, int optname, void* optval, socklen_t optlen);
+
   SocketCore(sock_t sockfd, int sockType);
 public:
   SocketCore(int sockType = SOCK_STREAM);
@@ -198,7 +177,7 @@ public:
    * You must call beginListen() before calling this method.
    * @return accepted socket.
    */
-  SharedHandle<SocketCore> acceptConnection() const;
+  std::shared_ptr<SocketCore> acceptConnection() const;
 
   /**
    * Connects to the server named host and the destination port is port.
@@ -328,8 +307,8 @@ public:
   bool wantWrite() const;
 
 #ifdef ENABLE_SSL
-  static void setClientTLSContext(const SharedHandle<TLSContext>& tlsContext);
-  static void setServerTLSContext(const SharedHandle<TLSContext>& tlsContext);
+  static void setClientTLSContext(const std::shared_ptr<TLSContext>& tlsContext);
+  static void setServerTLSContext(const std::shared_ptr<TLSContext>& tlsContext);
 #endif // ENABLE_SSL
 
   static void setProtocolFamily(int protocolFamily)
@@ -401,6 +380,12 @@ bool verifyHostname(const std::string& hostname,
                     const std::vector<std::string>& dnsNames,
                     const std::vector<std::string>& ipAddrs,
                     const std::string& commonName);
+// Checks public IP address are configured for each family: IPv4 and
+// IPv6. The result can be obtained using getIpv4AddrConfigured() and
+// getIpv6AddrConfigured() respectively.
+void checkAddrconfig();
+bool getIPv4AddrConfigured();
+bool getIPv6AddrConfigured();
 
 } // namespace net
 
