@@ -3,7 +3,6 @@
 #include <cstring>
 #include <cppunit/extensions/HelperMacros.h>
 
-#include "ARC4Decryptor.h"
 #include "Exception.h"
 #include "util.h"
 
@@ -12,23 +11,23 @@ namespace aria2 {
 class ARC4Test:public CppUnit::TestFixture {
 
   CPPUNIT_TEST_SUITE(ARC4Test);
-  CPPUNIT_TEST(testEncryptDecrypt);
+  CPPUNIT_TEST(testEncrypt);
   CPPUNIT_TEST_SUITE_END();
 public:
   void setUp() {}
 
   void tearDown() {}
 
-  void testEncryptDecrypt();
+  void testEncrypt();
 };
 
 
 CPPUNIT_TEST_SUITE_REGISTRATION(ARC4Test);
 
-void ARC4Test::testEncryptDecrypt()
+void ARC4Test::testEncrypt()
 {
   ARC4Encryptor enc;
-  ARC4Decryptor dec;
+  ARC4Encryptor dec;
   const size_t LEN = 20;
   unsigned char key[LEN];
   memset(key, 0, LEN);
@@ -38,13 +37,13 @@ void ARC4Test::testEncryptDecrypt()
 
   unsigned char encrypted[LEN];
   unsigned char decrypted[LEN];
-  enc.encrypt(encrypted, LEN, key, LEN);
-  dec.decrypt(decrypted, LEN, encrypted, LEN);
+  enc.encrypt(LEN, encrypted, key);
+  dec.encrypt(LEN, decrypted, encrypted);
 
   CPPUNIT_ASSERT(memcmp(key, decrypted, LEN) == 0);
   // once more
-  enc.encrypt(encrypted, LEN, key, LEN);
-  dec.decrypt(decrypted, LEN, encrypted, LEN);
+  enc.encrypt(LEN, encrypted, key);
+  dec.encrypt(LEN, decrypted, encrypted);
 
   CPPUNIT_ASSERT(memcmp(key, decrypted, LEN) == 0);
 }
