@@ -28,6 +28,7 @@ class BitfieldManTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testGetSparseMissingUnusedIndex_withMinSplitSize);
   CPPUNIT_TEST(testIsBitSetOffsetRange);
   CPPUNIT_TEST(testGetOffsetCompletedLength);
+  CPPUNIT_TEST(testGetOffsetCompletedLength_largeFile);
   CPPUNIT_TEST(testGetMissingUnusedLength);
   CPPUNIT_TEST(testSetBitRange);
   CPPUNIT_TEST(testGetAllMissingIndexes);
@@ -49,7 +50,7 @@ public:
   void testGetAllMissingIndexes_noarg();
   void testGetAllMissingIndexes_checkLastByte();
   void testGetAllMissingUnusedIndexes();
-  
+
   void testIsAllBitSet();
   void testFilter();
   void testIsFilterBitSet();
@@ -62,6 +63,7 @@ public:
   void testGetSparseMissingUnusedIndex_withMinSplitSize();
   void testIsBitSetOffsetRange();
   void testGetOffsetCompletedLength();
+  void testGetOffsetCompletedLength_largeFile();
   void testGetMissingUnusedLength();
   void testSetBitRange();
   void testCountFilteredBlock();
@@ -196,7 +198,7 @@ void BitfieldManTest::testIsAllBitSet() {
   CPPUNIT_ASSERT(!bt1.isAllBitSet());
   bt1.setBit(1);
   CPPUNIT_ASSERT(!bt1.isAllBitSet());
-  
+
   for(size_t i = 0; i < 8; i++) {
     CPPUNIT_ASSERT(bt1.setBit(i));
   }
@@ -426,7 +428,7 @@ void BitfieldManTest::testGetSparseMissingUnusedIndex_withMinSplitSize()
   CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)5, index);
-  bitfield.setBit(5);  
+  bitfield.setBit(5);
   CPPUNIT_ASSERT(!bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                        ignoreBitfield, length));
 }
@@ -449,11 +451,11 @@ void BitfieldManTest::testIsBitSetOffsetRange()
 
   bitfield.setBit(100);
   bitfield.setBit(101);
-  
+
   CPPUNIT_ASSERT(bitfield.isBitSetOffsetRange(pieceLength*100, pieceLength*2));
   CPPUNIT_ASSERT(!bitfield.isBitSetOffsetRange(pieceLength*100-10, pieceLength*2));
   CPPUNIT_ASSERT(!bitfield.isBitSetOffsetRange(pieceLength*100, pieceLength*2+1));
-    
+
   bitfield.clearAllBit();
 
   bitfield.setBit(100);
@@ -484,6 +486,26 @@ void BitfieldManTest::testGetOffsetCompletedLength()
   CPPUNIT_ASSERT_EQUAL((int64_t)0, bt.getOffsetCompletedLength(1024*20, 1));
 }
 
+void BitfieldManTest::testGetOffsetCompletedLength_largeFile()
+{
+  // Test for overflow on 32-bit systems.
+
+  // Total 4TiB, 4MiB block
+  BitfieldMan bt(1 << 22, 1LL << 40);
+  bt.setBit(1 << 11);
+  bt.setBit((1 << 11)+1);
+  bt.setBit((1 << 11)+2);
+
+  // The last piece is missing:
+  CPPUNIT_ASSERT_EQUAL((int64_t)bt.getBlockLength()*3,
+                       bt.getOffsetCompletedLength(1LL << 33, 1 << 24));
+
+  // The first piece is missing:
+  CPPUNIT_ASSERT_EQUAL((int64_t)bt.getBlockLength()*3,
+                       bt.getOffsetCompletedLength
+		       ((1LL << 33) - bt.getBlockLength(), 1 << 24));
+}
+
 void BitfieldManTest::testGetMissingUnusedLength()
 {
   int64_t totalLength = 1024*10+10;
@@ -668,7 +690,7 @@ void BitfieldManTest::testGetFirstNMissingUnusedIndex()
   CPPUNIT_ASSERT_EQUAL((size_t)3, out.size());
   for(size_t i = 0; i < out.size(); ++i) {
     CPPUNIT_ASSERT_EQUAL(ans[i], out[i]);
-  }  
+  }
   CPPUNIT_ASSERT_EQUAL((size_t)0, bt.getFirstNMissingUnusedIndex(out, 0));
   bt.setAllBit();
   CPPUNIT_ASSERT_EQUAL((size_t)0, bt.getFirstNMissingUnusedIndex(out, 10));
